package java.util.concurrent;

import java.util.NoSuchElementException;
import java.lang.ref.WeakReference;
import java.io.IOException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.util.Spliterators;
import java.util.Spliterator;
import java.lang.reflect.Array;
import java.util.Iterator;
import java.util.Collection;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;
import java.io.Serializable;
import java.util.AbstractQueue;

public class ArrayBlockingQueue<E> extends AbstractQueue<E> implements BlockingQueue<E>, Serializable
{
    private static final long serialVersionUID = -817911632652898426L;
    final Object[] items;
    int takeIndex;
    int putIndex;
    int count;
    final ReentrantLock lock;
    private final Condition notEmpty;
    private final Condition notFull;
    transient ArrayBlockingQueue.Itrs itrs;
    
    final int dec(int n) {
        return ((n == 0) ? this.items.length : n) - 1;
    }
    
    final E itemAt(int n) {
        return this.items[n];
    }
    
    private static void checkNotNull(Object o) {
        if (o == null) {
            throw new NullPointerException();
        }
        return;
    }
    
    private void enqueue(E e) {
        Object[] items = this.items;
        items[this.putIndex] = e;
        if (++this.putIndex == items.length) {
            this.putIndex = 0;
        }
        ++this.count;
        this.notEmpty.signal();
    }
    
    private E dequeue() {
        Object[] items = this.items;
        Object o = items[this.takeIndex];
        items[this.takeIndex] = null;
        if (++this.takeIndex == items.length) {
            this.takeIndex = 0;
        }
        --this.count;
        if (this.itrs != null) {
            this.itrs.elementDequeued();
        }
        this.notFull.signal();
        return o;
    }
    
    void removeAt(int putIndex) {
        Object[] items = this.items;
        if (putIndex == this.takeIndex) {
            items[this.takeIndex] = null;
            if (++this.takeIndex == items.length) {
                this.takeIndex = 0;
            }
            --this.count;
            if (this.itrs != null) {
                this.itrs.elementDequeued();
            }
        }
        else {
            int putIndex2 = this.putIndex;
            while (true) {
                int n = putIndex + 1;
                if (n == items.length) {
                    n = 0;
                }
                if (n != putIndex2) {
                    items[putIndex] = items[n];
                    putIndex = n;
                }
                else {
                    break;
                }
            }
            items[putIndex] = null;
            this.putIndex = putIndex;
            --this.count;
            if (this.itrs != null) {
                this.itrs.removedAt(putIndex);
            }
        }
        this.notFull.signal();
    }
    
    public ArrayBlockingQueue(int n) {
        this(n, false);
    }
    
    public ArrayBlockingQueue(int n, boolean fair) {
        this.itrs = null;
        if (n <= 0) {
            throw new IllegalArgumentException();
        }
        this.items = new Object[n];
        this.lock = new ReentrantLock(fair);
        this.notEmpty = this.lock.newCondition();
        this.notFull = this.lock.newCondition();
        return;
    }
    
    public ArrayBlockingQueue(int n, boolean b, Collection<? extends E> collection) {
        this(n, b);
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            int count = 0;
            try {
                for (Object next : collection) {
                    checkNotNull(next);
                    this.items[count++] = next;
                }
            }
            catch (ArrayIndexOutOfBoundsException ex) {
                throw new IllegalArgumentException();
            }
            this.count = count;
            this.putIndex = ((count == n) ? 0 : count);
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public boolean add(E e) {
        return super.add(e);
    }
    
    @Override
    public boolean offer(E e) {
        checkNotNull(e);
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            if (this.count == this.items.length) {
                return false;
            }
            this.enqueue(e);
            return true;
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public void put(E e) throws InterruptedException {
        checkNotNull(e);
        ReentrantLock lock = this.lock;
        lock.lockInterruptibly();
        try {
            while (this.count == this.items.length) {
                this.notFull.await();
            }
            this.enqueue(e);
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public boolean offer(E e, long n, TimeUnit timeUnit) throws InterruptedException {
        checkNotNull(e);
        long n2 = timeUnit.toNanos(n);
        ReentrantLock lock = this.lock;
        lock.lockInterruptibly();
        try {
            while (this.count == this.items.length) {
                if (n2 <= 0L) {
                    return false;
                }
                n2 = this.notFull.awaitNanos(n2);
            }
            this.enqueue(e);
            return true;
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public E poll() {
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            return (this.count == 0) ? null : this.dequeue();
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public E take() throws InterruptedException {
        ReentrantLock lock = this.lock;
        lock.lockInterruptibly();
        try {
            while (this.count == 0) {
                this.notEmpty.await();
            }
            return this.dequeue();
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public E poll(long n, TimeUnit timeUnit) throws InterruptedException {
        long n2 = timeUnit.toNanos(n);
        ReentrantLock lock = this.lock;
        lock.lockInterruptibly();
        try {
            while (this.count == 0) {
                if (n2 <= 0L) {
                    return null;
                }
                n2 = this.notEmpty.awaitNanos(n2);
            }
            return this.dequeue();
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public E peek() {
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            return this.itemAt(this.takeIndex);
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public int size() {
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            return this.count;
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public int remainingCapacity() {
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            return this.items.length - this.count;
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public boolean remove(Object o) {
        if (o == null) {
            return false;
        }
        Object[] items = this.items;
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            Label_0086: {
                if (this.count > 0) {
                    int putIndex = this.putIndex;
                    int takeIndex = this.takeIndex;
                    while (!o.equals(items[takeIndex])) {
                        if (++takeIndex == items.length) {
                            takeIndex = 0;
                        }
                        if (takeIndex == putIndex) {
                            break Label_0086;
                        }
                        continue;
                    }
                    this.removeAt(takeIndex);
                    return true;
                }
            }
            return false;
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public boolean contains(Object o) {
        if (o == null) {
            return false;
        }
        Object[] items = this.items;
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            Label_0080: {
                if (this.count > 0) {
                    int putIndex = this.putIndex;
                    int takeIndex = this.takeIndex;
                    while (!o.equals(items[takeIndex])) {
                        if (++takeIndex == items.length) {
                            takeIndex = 0;
                        }
                        if (takeIndex == putIndex) {
                            break Label_0080;
                        }
                        continue;
                    }
                    return true;
                }
            }
            return false;
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public Object[] toArray() {
        ReentrantLock lock = this.lock;
        lock.lock();
        Object[] array;
        try {
            int count = this.count;
            array = new Object[count];
            int n = this.items.length - this.takeIndex;
            if (count <= n) {
                System.arraycopy(this.items, this.takeIndex, array, 0, count);
            }
            else {
                System.arraycopy(this.items, this.takeIndex, array, 0, n);
                System.arraycopy(this.items, 0, array, n, count - n);
            }
        }
        finally {
            lock.unlock();
        }
        return array;
    }
    
    @Override
    public <T> T[] toArray(T[] array) {
        Object[] items = this.items;
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            int count = this.count;
            int length = array.length;
            if (length < count) {
                array = (Object[])(Object[])Array.newInstance(array.getClass().getComponentType(), count);
            }
            int n = items.length - this.takeIndex;
            if (count <= n) {
                System.arraycopy(items, this.takeIndex, array, 0, count);
            }
            else {
                System.arraycopy(items, this.takeIndex, array, 0, n);
                System.arraycopy(items, 0, array, n, count - n);
            }
            if (length > count) {
                array[count] = null;
            }
        }
        finally {
            lock.unlock();
        }
        return array;
    }
    
    @Override
    public String toString() {
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            int count = this.count;
            if (count == 0) {
                return "[]";
            }
            Object[] items = this.items;
            StringBuilder sb = new StringBuilder();
            sb.append('[');
            int takeIndex = this.takeIndex;
            while (true) {
                Object o = items[takeIndex];
                sb.append((o == this) ? "(this Collection)" : o);
                if (--count == 0) {
                    break;
                }
                sb.append(',').append(' ');
                if (++takeIndex == items.length) {
                    takeIndex = 0;
                }
                else {
                    continue;
                }
            }
            return sb.append(']').toString();
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public void clear() {
        Object[] items = this.items;
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            int count = this.count;
            if (count > 0) {
                int putIndex = this.putIndex;
                int i = this.takeIndex;
                do {
                    items[i] = null;
                    if (++i == items.length) {
                        i = 0;
                    }
                    else {
                        continue;
                    }
                } while (i != putIndex);
                this.takeIndex = putIndex;
                this.count = 0;
                if (this.itrs != null) {
                    this.itrs.queueIsEmpty();
                }
                while (count > 0 && lock.hasWaiters(this.notFull)) {
                    this.notFull.signal();
                    --count;
                }
            }
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public int drainTo(Collection<? super E> collection) {
        return this.drainTo(collection, Integer.MAX_VALUE);
    }
    
    @Override
    public int drainTo(Collection<? super E> collection, int a) {
        checkNotNull(collection);
        if (collection == this) {
            throw new IllegalArgumentException();
        }
        if (a <= 0) {
            return 0;
        }
        Object[] items = this.items;
        ReentrantLock lock = this.lock;
        lock.lock();
        try {
            int min = Math.min(a, this.count);
            int takeIndex = this.takeIndex;
            int i = 0;
            try {
                while (i < min) {
                    collection.add(items[takeIndex]);
                    items[takeIndex] = null;
                    if (++takeIndex == items.length) {
                        takeIndex = 0;
                    }
                    ++i;
                }
                if (i > 0) {
                    this.count -= i;
                    this.takeIndex = takeIndex;
                    if (this.itrs != null) {
                        if (this.count == 0) {
                            this.itrs.queueIsEmpty();
                        }
                        else if (i > takeIndex) {
                            this.itrs.takeIndexWrapped();
                        }
                    }
                    while (i > 0 && lock.hasWaiters(this.notFull)) {
                        this.notFull.signal();
                        --i;
                    }
                }
                return min;
            }
            finally {
                if (i > 0) {
                    this.count -= i;
                    this.takeIndex = takeIndex;
                    if (this.itrs != null) {
                        if (this.count == 0) {
                            this.itrs.queueIsEmpty();
                        }
                        else if (i > takeIndex) {
                            this.itrs.takeIndexWrapped();
                        }
                    }
                    while (i > 0 && lock.hasWaiters(this.notFull)) {
                        this.notFull.signal();
                        --i;
                    }
                }
            }
        }
        finally {
            lock.unlock();
        }
    }
    
    @Override
    public Iterator<E> iterator() {
        return new ArrayBlockingQueue.Itr();
    }
    
    @Override
    public Spliterator<E> spliterator() {
        return Spliterators.spliterator(this, 4368);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (this.items.length == 0 || this.takeIndex < 0 || this.takeIndex >= this.items.length || this.putIndex < 0 || this.putIndex >= this.items.length || this.count < 0 || this.count > this.items.length || Math.floorMod(this.putIndex - this.takeIndex, this.items.length) != Math.floorMod(this.count, this.items.length)) {
            throw new InvalidObjectException("invariants violated");
        }
        return;
    }
    
    class Itrs
    {
        int cycles;
        private ArrayBlockingQueue.Itrs.Node head;
        private ArrayBlockingQueue.Itrs.Node sweeper;
        private static final int SHORT_SWEEP_PROBES = 4;
        private static final int LONG_SWEEP_PROBES = 16;
        
        Itrs(ArrayBlockingQueue.Itr itr) {
            this.cycles = 0;
            this.sweeper = null;
            this.register(itr);
        }
        
        void doSomeSweeping(boolean b) {
            int i = b ? 16 : 4;
            ArrayBlockingQueue.Itrs.Node sweeper = this.sweeper;
            ArrayBlockingQueue.Itrs.Node node;
            ArrayBlockingQueue.Itrs.Node node2;
            int n;
            if (sweeper == null) {
                node = null;
                node2 = this.head;
                n = 1;
            }
            else {
                node = sweeper;
                node2 = node.next;
                n = 0;
            }
            while (i > 0) {
                if (node2 == null) {
                    if (n != 0) {
                        break;
                    }
                    node = null;
                    node2 = this.head;
                    n = 1;
                }
                ArrayBlockingQueue.Itr itr = (ArrayBlockingQueue.Itr)node2.get();
                ArrayBlockingQueue.Itrs.Node next = node2.next;
                if (itr == null || itr.isDetached()) {
                    i = 16;
                    node2.clear();
                    node2.next = null;
                    if (node == null) {
                        if ((this.head = next) == null) {
                            ArrayBlockingQueue.this.itrs = null;
                            return;
                        }
                    }
                    else {
                        node.next = next;
                    }
                }
                else {
                    node = node2;
                }
                node2 = next;
                --i;
            }
            this.sweeper = ((node2 == null) ? null : node);
        }
        
        void register(ArrayBlockingQueue.Itr itr) {
            this.head = new ArrayBlockingQueue.Itrs.Node(itr, this.head);
        }
        
        void takeIndexWrapped() {
            ++this.cycles;
            ArrayBlockingQueue.Itrs.Node node = null;
            ArrayBlockingQueue.Itrs.Node next;
            for (ArrayBlockingQueue.Itrs.Node head = this.head; head != null; head = next) {
                ArrayBlockingQueue.Itr itr = (ArrayBlockingQueue.Itr)head.get();
                next = head.next;
                if (itr == null || itr.takeIndexWrapped()) {
                    head.clear();
                    head.next = null;
                    if (node == null) {
                        this.head = next;
                    }
                    else {
                        node.next = next;
                    }
                }
                else {
                    node = head;
                }
            }
            if (this.head == null) {
                ArrayBlockingQueue.this.itrs = null;
            }
        }
        
        void removedAt(int n) {
            ArrayBlockingQueue.Itrs.Node node = null;
            ArrayBlockingQueue.Itrs.Node next;
            for (ArrayBlockingQueue.Itrs.Node head = this.head; head != null; head = next) {
                ArrayBlockingQueue.Itr itr = (ArrayBlockingQueue.Itr)head.get();
                next = head.next;
                if (itr == null || itr.removedAt(n)) {
                    head.clear();
                    head.next = null;
                    if (node == null) {
                        this.head = next;
                    }
                    else {
                        node.next = next;
                    }
                }
                else {
                    node = head;
                }
            }
            if (this.head == null) {
                ArrayBlockingQueue.this.itrs = null;
            }
        }
        
        void queueIsEmpty() {
            for (ArrayBlockingQueue.Itrs.Node node = this.head; node != null; node = node.next) {
                ArrayBlockingQueue.Itr itr = (ArrayBlockingQueue.Itr)node.get();
                if (itr != null) {
                    node.clear();
                    itr.shutdown();
                }
            }
            this.head = null;
            ArrayBlockingQueue.this.itrs = null;
        }
        
        void elementDequeued() {
            if (ArrayBlockingQueue.this.count == 0) {
                this.queueIsEmpty();
            }
            else if (ArrayBlockingQueue.this.takeIndex == 0) {
                this.takeIndexWrapped();
            }
        }
        
        private class Node extends WeakReference<ArrayBlockingQueue.Itr>
        {
            ArrayBlockingQueue.Itrs.Node next;
            
            Node(ArrayBlockingQueue.Itr referent, ArrayBlockingQueue.Itrs.Node next) {
                super(referent);
                this.next = next;
            }
        }
    }
    
    private class Itr implements Iterator<E>
    {
        private int cursor;
        private E nextItem;
        private int nextIndex;
        private E lastItem;
        private int lastRet;
        private int prevTakeIndex;
        private int prevCycles;
        private static final int NONE = -1;
        private static final int REMOVED = -2;
        private static final int DETACHED = -3;
        
        Itr() {
            this.lastRet = -1;
            ReentrantLock lock = ArrayBlockingQueue.this.lock;
            lock.lock();
            try {
                if (ArrayBlockingQueue.this.count == 0) {
                    this.cursor = -1;
                    this.nextIndex = -1;
                    this.prevTakeIndex = -3;
                }
                else {
                    int takeIndex = ArrayBlockingQueue.this.takeIndex;
                    this.prevTakeIndex = takeIndex;
                    int nextIndex = takeIndex;
                    this.nextIndex = nextIndex;
                    this.nextItem = ArrayBlockingQueue.this.itemAt(nextIndex);
                    this.cursor = this.incCursor(takeIndex);
                    if (ArrayBlockingQueue.this.itrs == null) {
                        ArrayBlockingQueue.this.itrs = new ArrayBlockingQueue.Itrs(this);
                    }
                    else {
                        ArrayBlockingQueue.this.itrs.register(this);
                        ArrayBlockingQueue.this.itrs.doSomeSweeping(false);
                    }
                    this.prevCycles = ArrayBlockingQueue.this.itrs.cycles;
                }
            }
            finally {
                lock.unlock();
            }
        }
        
        boolean isDetached() {
            return this.prevTakeIndex < 0;
        }
        
        private int incCursor(int n) {
            if (++n == ArrayBlockingQueue.this.items.length) {
                n = 0;
            }
            if (n == ArrayBlockingQueue.this.putIndex) {
                n = -1;
            }
            return n;
        }
        
        private boolean invalidated(int n, int n2, long n3, int n4) {
            if (n < 0) {
                return false;
            }
            int n5 = n - n2;
            if (n5 < 0) {
                n5 += n4;
            }
            return n3 > (long)n5;
        }
        
        private void incorporateDequeues() {
            int cycles = ArrayBlockingQueue.this.itrs.cycles;
            int takeIndex = ArrayBlockingQueue.this.takeIndex;
            int prevCycles = this.prevCycles;
            int prevTakeIndex = this.prevTakeIndex;
            if (cycles != prevCycles || takeIndex != prevTakeIndex) {
                int length = ArrayBlockingQueue.this.items.length;
                long n = (long)((cycles - prevCycles) * length + (takeIndex - prevTakeIndex));
                if (this.invalidated(this.lastRet, prevTakeIndex, n, length)) {
                    this.lastRet = -2;
                }
                if (this.invalidated(this.nextIndex, prevTakeIndex, n, length)) {
                    this.nextIndex = -2;
                }
                if (this.invalidated(this.cursor, prevTakeIndex, n, length)) {
                    this.cursor = takeIndex;
                }
                if (this.cursor < 0 && this.nextIndex < 0 && this.lastRet < 0) {
                    this.detach();
                }
                else {
                    this.prevCycles = cycles;
                    this.prevTakeIndex = takeIndex;
                }
            }
        }
        
        private void detach() {
            if (this.prevTakeIndex >= 0) {
                this.prevTakeIndex = -3;
                ArrayBlockingQueue.this.itrs.doSomeSweeping(true);
            }
        }
        
        @Override
        public boolean hasNext() {
            if (this.nextItem != null) {
                return true;
            }
            this.noNext();
            return false;
        }
        
        private void noNext() {
            ReentrantLock lock = ArrayBlockingQueue.this.lock;
            lock.lock();
            try {
                if (!this.isDetached()) {
                    this.incorporateDequeues();
                    if (this.lastRet >= 0) {
                        this.lastItem = ArrayBlockingQueue.this.itemAt(this.lastRet);
                        this.detach();
                    }
                }
            }
            finally {
                lock.unlock();
            }
        }
        
        @Override
        public E next() {
            Object nextItem = this.nextItem;
            if (nextItem == null) {
                throw new NoSuchElementException();
            }
            ReentrantLock lock = ArrayBlockingQueue.this.lock;
            lock.lock();
            try {
                if (!this.isDetached()) {
                    this.incorporateDequeues();
                }
                this.lastRet = this.nextIndex;
                int cursor = this.cursor;
                if (cursor >= 0) {
                    ArrayBlockingQueue<Object> this$0 = ArrayBlockingQueue.this;
                    int nextIndex = cursor;
                    this.nextIndex = nextIndex;
                    this.nextItem = this$0.itemAt(nextIndex);
                    this.cursor = this.incCursor(cursor);
                }
                else {
                    this.nextIndex = -1;
                    this.nextItem = null;
                }
            }
            finally {
                lock.unlock();
            }
            return nextItem;
        }
        
        @Override
        public void remove() {
            ReentrantLock lock = ArrayBlockingQueue.this.lock;
            lock.lock();
            try {
                if (!this.isDetached()) {
                    this.incorporateDequeues();
                }
                int lastRet = this.lastRet;
                this.lastRet = -1;
                if (lastRet >= 0) {
                    if (!this.isDetached()) {
                        ArrayBlockingQueue.this.removeAt(lastRet);
                    }
                    else {
                        Object lastItem = this.lastItem;
                        this.lastItem = null;
                        if (ArrayBlockingQueue.this.itemAt(lastRet) == lastItem) {
                            ArrayBlockingQueue.this.removeAt(lastRet);
                        }
                    }
                }
                else if (lastRet == -1) {
                    throw new IllegalStateException();
                }
                if (this.cursor < 0 && this.nextIndex < 0) {
                    this.detach();
                }
            }
            finally {
                lock.unlock();
            }
        }
        
        void shutdown() {
            this.cursor = -1;
            if (this.nextIndex >= 0) {
                this.nextIndex = -2;
            }
            if (this.lastRet >= 0) {
                this.lastRet = -2;
                this.lastItem = null;
            }
            this.prevTakeIndex = -3;
        }
        
        private int distance(int n, int n2, int n3) {
            int n4 = n - n2;
            if (n4 < 0) {
                n4 += n3;
            }
            return n4;
        }
        
        boolean removedAt(int n) {
            if (this.isDetached()) {
                return true;
            }
            int cycles = ArrayBlockingQueue.this.itrs.cycles;
            int takeIndex = ArrayBlockingQueue.this.takeIndex;
            int prevCycles = this.prevCycles;
            int prevTakeIndex = this.prevTakeIndex;
            int length = ArrayBlockingQueue.this.items.length;
            int n2 = cycles - prevCycles;
            if (n < takeIndex) {
                ++n2;
            }
            int n3 = n2 * length + (n - prevTakeIndex);
            int cursor = this.cursor;
            if (cursor >= 0) {
                int distance = this.distance(cursor, prevTakeIndex, length);
                if (distance == n3) {
                    if (cursor == ArrayBlockingQueue.this.putIndex) {
                        cursor = (this.cursor = -1);
                    }
                }
                else if (distance > n3) {
                    cursor = (this.cursor = ArrayBlockingQueue.this.dec(cursor));
                }
            }
            int lastRet = this.lastRet;
            if (lastRet >= 0) {
                int distance2 = this.distance(lastRet, prevTakeIndex, length);
                if (distance2 == n3) {
                    lastRet = (this.lastRet = -2);
                }
                else if (distance2 > n3) {
                    lastRet = (this.lastRet = ArrayBlockingQueue.this.dec(lastRet));
                }
            }
            int nextIndex = this.nextIndex;
            if (nextIndex >= 0) {
                int distance3 = this.distance(nextIndex, prevTakeIndex, length);
                if (distance3 == n3) {
                    this.nextIndex = -2;
                }
                else if (distance3 > n3) {
                    this.nextIndex = ArrayBlockingQueue.this.dec(nextIndex);
                }
            }
            else if (cursor < 0 && nextIndex < 0 && lastRet < 0) {
                this.prevTakeIndex = -3;
                return true;
            }
            return false;
        }
        
        boolean takeIndexWrapped() {
            if (this.isDetached()) {
                return true;
            }
            if (ArrayBlockingQueue.this.itrs.cycles - this.prevCycles > 1) {
                this.shutdown();
                return true;
            }
            return false;
        }
    }
}
